Package: @PKGNAME@
Title: @TITLE@
Version: @VERSION@
Author: @AUTHOR@
Description: Data Package automatically created on @TODAY@.
Maintainer: @MAINTAINER@
Depends: Biobase
License:  @LICENSE@
biocViews: @BVIEWS@
