Package: @PKGNAME@
Title: Greek mythology package
Version: @VERSION@
Author: @AUTHOR@
Description: Package automatically created on @TODAY@.
Maintainer: Some One <@AUTHOR@@mtolympus.gr>
License:  LGPL
