aaMap <- structure(list(name = structure(c(1, 5, 4, 6, 14, 8, 9, 10, 12, 
11, 13, 3, 15, 7, 2, 16, 17, 20, 18, 19), .Label = c("alanine", 
"arginine", "asparagine", "aspartic.acid", "cysteine", "glutamic.acid", 
"glutamine", "glycine", "histidine", "isoleucine", "leucine", 
"lysine", "methionine", "phenylalanine", "proline", "serine", 
"threonine", "tryptophan", "tyrosine", "valine"), class = "factor"), 
    let.1 = structure(c(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 
    13, 14, 15, 16, 17, 18, 19, 20), .Label = c("A", "C", "D", 
    "E", "F", "G", "H", "I", "K", "L", "M", "N", "P", "Q", "R", 
    "S", "T", "V", "W", "Y"), class = "factor"), let.3 = structure(c(1, 
    5, 4, 7, 14, 8, 9, 10, 12, 11, 13, 3, 15, 6, 2, 16, 17, 20, 
    18, 19), .Label = c("ala", "arg", "asn", "asp", "cys", "gln", 
    "glu", "gly", "his", "ile", "leu", "lys", "met", "phe", "pro", 
    "ser", "thr", "trp", "tyr", "val"), class = "factor"), scProp = structure(c(1, 
    2, 2, 2, 1, 1, 2, 1, 2, 1, 1, 2, 1, 2, 2, 2, 2, 1, 1, 2), .Label = c("nonpolar", 
    "polar"), class = "factor"), hyPhilic = c(FALSE, NA, TRUE, 
    TRUE, FALSE, NA, TRUE, FALSE, TRUE, FALSE, FALSE, TRUE, NA, 
    TRUE, TRUE, NA, NA, FALSE, NA, NA), acidic = c(NA, NA, TRUE, 
    TRUE, NA, NA, FALSE, NA, FALSE, NA, NA, FALSE, NA, FALSE, 
    FALSE, NA, NA, NA, NA, NA)), .Names = c("name", "let.1", 
"let.3", "scProp", "hyPhilic", "acidic"), class = "data.frame", row.names = c("1", 
"2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", 
"14", "15", "16", "17", "18", "19", "20"))
